'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const core = require('./core-51053920.js');

/* tslint:disable:no-bitwise */
class Base64 {
    // base64 is 4/3 + up to two characters of the original data
    static byteLength(b64) {
        const [validLength, placeHoldersLength] = Base64._getLengths(b64);
        return Base64._byteLength(validLength, placeHoldersLength);
    }
    static decode(b64) {
        Base64._initRevLookup();
        const [validLength, placeHoldersLength] = Base64._getLengths(b64);
        const arr = new Uint8Array(Base64._byteLength(validLength, placeHoldersLength));
        let curByte = 0;
        // if there are placeholders, only get up to the last complete 4 chars
        const len = placeHoldersLength > 0 ? validLength - 4 : validLength;
        let i = 0;
        for (; i < len; i += 4) {
            const tmp = (Base64._revLookup[b64.charCodeAt(i)] << 18) |
                (Base64._revLookup[b64.charCodeAt(i + 1)] << 12) |
                (Base64._revLookup[b64.charCodeAt(i + 2)] << 6) |
                Base64._revLookup[b64.charCodeAt(i + 3)];
            arr[curByte++] = (tmp >> 16) & 0xFF;
            arr[curByte++] = (tmp >> 8) & 0xFF;
            arr[curByte++] = tmp & 0xFF;
        }
        if (placeHoldersLength === 2) {
            const tmp = (Base64._revLookup[b64.charCodeAt(i)] << 2) |
                (Base64._revLookup[b64.charCodeAt(i + 1)] >> 4);
            arr[curByte++] = tmp & 0xFF;
        }
        if (placeHoldersLength === 1) {
            const tmp = (Base64._revLookup[b64.charCodeAt(i)] << 10) |
                (Base64._revLookup[b64.charCodeAt(i + 1)] << 4) |
                (Base64._revLookup[b64.charCodeAt(i + 2)] >> 2);
            arr[curByte++] = (tmp >> 8) & 0xFF;
            arr[curByte /*++ not needed*/] = tmp & 0xFF;
        }
        return arr;
    }
    static encode(uint8) {
        const length = uint8.length;
        const extraBytes = length % 3; // if we have 1 byte left, pad 2 bytes
        const parts = [];
        const maxChunkLength = 16383; // must be multiple of 3
        // go through the array every three bytes, we'll deal with trailing stuff later
        for (let i = 0, len2 = length - extraBytes; i < len2; i += maxChunkLength) {
            parts.push(Base64._encodeChunk(uint8, i, (i + maxChunkLength) > len2 ? len2 : (i + maxChunkLength)));
        }
        // pad the end with zeros, but make sure to not forget the extra bytes
        if (extraBytes === 1) {
            const tmp = uint8[length - 1];
            parts.push(Base64._lookup[tmp >> 2] +
                Base64._lookup[(tmp << 4) & 0x3F] +
                '==');
        }
        else if (extraBytes === 2) {
            const tmp = (uint8[length - 2] << 8) + uint8[length - 1];
            parts.push(Base64._lookup[tmp >> 10] +
                Base64._lookup[(tmp >> 4) & 0x3F] +
                Base64._lookup[(tmp << 2) & 0x3F] +
                '=');
        }
        return parts.join('');
    }
    static encodeUrl(buffer) {
        return Base64.encode(buffer).replace(/\//g, '_').replace(/\+/g, '-').replace(/=/g, '.');
    }
    static decodeUrl(base64) {
        return Base64.decode(base64.replace(/_/g, '/').replace(/-/g, '+').replace(/\./g, '='));
    }
    static _initRevLookup() {
        if (Base64._revLookup.length !== 0)
            return;
        Base64._revLookup = [];
        for (let i = 0, len = Base64._lookup.length; i < len; i++) {
            Base64._revLookup[Base64._lookup.charCodeAt(i)] = i;
        }
        // Support decoding URL-safe base64 strings, as Node.js does.
        // See: https://en.wikipedia.org/wiki/Base64#URL_applications
        Base64._revLookup['-'.charCodeAt(0)] = 62;
        Base64._revLookup['_'.charCodeAt(0)] = 63;
    }
    static _getLengths(b64) {
        const length = b64.length;
        if (length % 4 > 0) {
            throw new Error('Invalid string. Length must be a multiple of 4');
        }
        // Trim off extra bytes after placeholder bytes are found
        // See: https://github.com/beatgammit/base64-js/issues/42
        let validLength = b64.indexOf('=');
        if (validLength === -1)
            validLength = length;
        const placeHoldersLength = validLength === length ? 0 : 4 - (validLength % 4);
        return [validLength, placeHoldersLength];
    }
    static _byteLength(validLength, placeHoldersLength) {
        return ((validLength + placeHoldersLength) * 3 / 4) - placeHoldersLength;
    }
    static _tripletToBase64(num) {
        return Base64._lookup[num >> 18 & 0x3F] +
            Base64._lookup[num >> 12 & 0x3F] +
            Base64._lookup[num >> 6 & 0x3F] +
            Base64._lookup[num & 0x3F];
    }
    static _encodeChunk(uint8, start, end) {
        const output = [];
        for (let i = start; i < end; i += 3) {
            const tmp = ((uint8[i] << 16) & 0xFF0000) +
                ((uint8[i + 1] << 8) & 0xFF00) +
                (uint8[i + 2] & 0xFF);
            output.push(Base64._tripletToBase64(tmp));
        }
        return output.join('');
    }
}
Base64._lookup = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
Base64._revLookup = [];

var ExtraJSONTypes;
(function (ExtraJSONTypes) {
    ExtraJSONTypes[ExtraJSONTypes["UINT8_ARRAY"] = 0] = "UINT8_ARRAY";
})(ExtraJSONTypes || (ExtraJSONTypes = {}));
class JSONUtils {
    static stringify(value) {
        return JSON.stringify(value, JSONUtils._jsonifyType);
    }
    static parse(value) {
        return JSON.parse(value, JSONUtils._parseType);
    }
    static _parseType(key, value) {
        if (value && value.hasOwnProperty &&
            value.hasOwnProperty(JSONUtils.TYPE_SYMBOL) && value.hasOwnProperty(JSONUtils.VALUE_SYMBOL)) {
            switch (value[JSONUtils.TYPE_SYMBOL]) {
                case ExtraJSONTypes.UINT8_ARRAY:
                    return Base64.decode(value[JSONUtils.VALUE_SYMBOL]);
            }
        }
        return value;
    }
    static _jsonifyType(key, value) {
        if (value instanceof Uint8Array) {
            return JSONUtils._typedObject(ExtraJSONTypes.UINT8_ARRAY, Base64.encode(value));
        }
        return value;
    }
    static _typedObject(type, value) {
        const obj = {};
        obj[JSONUtils.TYPE_SYMBOL] = type;
        obj[JSONUtils.VALUE_SYMBOL] = value;
        return obj;
    }
}
JSONUtils.TYPE_SYMBOL = '__';
JSONUtils.VALUE_SYMBOL = 'v';

class RandomUtils {
    static generateRandomId() {
        const array = new Uint32Array(1);
        crypto.getRandomValues(array);
        return array[0];
    }
}

var ResponseStatus;
(function (ResponseStatus) {
    ResponseStatus["OK"] = "ok";
    ResponseStatus["ERROR"] = "error";
})(ResponseStatus || (ResponseStatus = {}));
const POSTMESSAGE_RETURN_URL = '<postMessage>';

class RequestIdStorage {
    /**
     * @param storeState Whether to store state in sessionStorage
     */
    constructor(storeState = true) {
        this._store = storeState ? window.sessionStorage : null;
        this._validIds = new Map();
        if (storeState) {
            this._restoreIds();
        }
    }
    static _decodeIds(ids) {
        const obj = JSONUtils.parse(ids);
        const validIds = new Map();
        for (const key of Object.keys(obj)) {
            const integerKey = parseInt(key, 10);
            validIds.set(isNaN(integerKey) ? key : integerKey, obj[key]);
        }
        return validIds;
    }
    has(id) {
        return this._validIds.has(id);
    }
    getCommand(id) {
        const result = this._validIds.get(id);
        return result ? result[0] : null;
    }
    getState(id) {
        const result = this._validIds.get(id);
        return result ? result[1] : null;
    }
    add(id, command, state = null) {
        this._validIds.set(id, [command, state]);
        this._storeIds();
    }
    remove(id) {
        this._validIds.delete(id);
        this._storeIds();
    }
    clear() {
        this._validIds.clear();
        if (this._store) {
            this._store.removeItem(RequestIdStorage.KEY);
        }
    }
    _encodeIds() {
        const obj = Object.create(null);
        for (const [key, value] of this._validIds) {
            obj[key] = value;
        }
        return JSONUtils.stringify(obj);
    }
    _restoreIds() {
        const requests = this._store.getItem(RequestIdStorage.KEY);
        if (requests) {
            this._validIds = RequestIdStorage._decodeIds(requests);
        }
    }
    _storeIds() {
        if (this._store) {
            this._store.setItem(RequestIdStorage.KEY, this._encodeIds());
        }
    }
}
RequestIdStorage.KEY = 'rpcRequests';

class UrlRpcEncoder {
    static receiveRedirectCommand(location) {
        const url = new URL(location.href);
        // Need referrer for origin check
        if (!document.referrer)
            return null;
        const referrer = new URL(document.referrer);
        // Parse query
        const params = new URLSearchParams(url.search);
        const fragment = new URLSearchParams(url.hash.substring(1));
        // Ignore messages without an ID
        if (!fragment.has('id'))
            return null;
        const id = parseInt(fragment.get('id'), 10);
        fragment.delete('id');
        params.set(UrlRpcEncoder.URL_SEARCHPARAM_NAME, id.toString());
        // Ignore messages without a command
        if (!fragment.has('command'))
            return null;
        const command = fragment.get('command');
        fragment.delete('command');
        // Ignore messages without a valid return path
        if (!fragment.has('returnURL'))
            return null;
        const returnURL = fragment.get('returnURL');
        fragment.delete('returnURL');
        const answerByPostMessage = returnURL === POSTMESSAGE_RETURN_URL
            && (window.opener || window.parent);
        // Only allow returning to same origin
        if (!answerByPostMessage && new URL(returnURL).origin !== referrer.origin)
            return null;
        // Parse args
        let args = [];
        if (fragment.has('args')) {
            try {
                args = JSONUtils.parse(fragment.get('args'));
            }
            catch (e) {
                // Do nothing
            }
        }
        args = Array.isArray(args) ? args : [];
        fragment.delete('args');
        url.search = params.toString();
        this._setUrlFragment(url, fragment);
        history.replaceState(history.state, '', url.href);
        return {
            origin: referrer.origin,
            data: {
                id,
                command,
                args,
            },
            returnURL,
            source: answerByPostMessage ? (window.opener || window.parent) : null,
        };
    }
    static receiveRedirectResponse(location) {
        const url = new URL(location.href);
        // Need referrer for origin check
        if (!document.referrer)
            return null;
        const referrer = new URL(document.referrer);
        // Parse query
        const params = new URLSearchParams(url.search);
        const fragment = new URLSearchParams(url.hash.substring(1));
        // Ignore messages without an ID
        if (!fragment.has('id'))
            return null;
        const id = parseInt(fragment.get('id'), 10);
        fragment.delete('id');
        params.set(UrlRpcEncoder.URL_SEARCHPARAM_NAME, id.toString());
        // Ignore messages without a status
        if (!fragment.has('status'))
            return null;
        const status = fragment.get('status') === ResponseStatus.OK ? ResponseStatus.OK : ResponseStatus.ERROR;
        fragment.delete('status');
        // Ignore messages without a result
        if (!fragment.has('result'))
            return null;
        const result = JSONUtils.parse(fragment.get('result'));
        fragment.delete('result');
        url.search = params.toString();
        this._setUrlFragment(url, fragment);
        history.replaceState(history.state, '', url.href);
        return {
            origin: referrer.origin,
            data: {
                id,
                status,
                result,
            },
        };
    }
    static prepareRedirectReply(state, status, result) {
        const returnUrl = new URL(state.returnURL);
        const fragment = new URLSearchParams(returnUrl.hash.substring(1));
        fragment.set('id', state.id.toString());
        fragment.set('status', status);
        fragment.set('result', JSONUtils.stringify(result));
        returnUrl.hash = fragment.toString();
        return returnUrl.href;
    }
    static prepareRedirectInvocation(targetURL, id, returnURL, command, args) {
        const targetUrl = new URL(targetURL);
        const fragment = new URLSearchParams(targetUrl.hash.substring(1));
        fragment.set('id', id.toString());
        fragment.set('returnURL', returnURL);
        fragment.set('command', command);
        if (Array.isArray(args)) {
            fragment.set('args', JSONUtils.stringify(args));
        }
        targetUrl.hash = fragment.toString();
        return targetUrl.href;
    }
    static _setUrlFragment(url, fragment) {
        /*
        The Url might include a fragment on its own before adding the parameters to it.
        It might even have a fragment consisting of other parameters.
        A '=' at the last position of the remaining fragment string indicates that at least one fragment
        part is remaining. Since URLSearchParams will try to represent a key=value pair with the value
        missing the '=' is added.
        Unfortunately fragments (as in regular fragment, not parameters) ending in a '=' will not
        work with this implementation. All other fragments, including other sets of parameters should be
        preserved by removing the '=' in case it exists at the end of the fragment. However, if other
        parameters are used without values (i.e. #abc&123&value) they will now include a '=' except
        for the last one (i.e. #abc=&123=&value), which is a valid input to URLSearchParams.
        */
        if (fragment.toString().endsWith('=')) {
            url.hash = fragment.toString().slice(0, -1);
        }
        else {
            url.hash = fragment.toString();
        }
    }
}
UrlRpcEncoder.URL_SEARCHPARAM_NAME = 'rpcId';

class RpcClient {
    constructor(allowedOrigin, storeState = false) {
        this._allowedOrigin = allowedOrigin;
        this._waitingRequests = new RequestIdStorage(storeState);
        this._responseHandlers = new Map();
        this._preserveRequests = false;
    }
    onResponse(command, resolve, reject) {
        this._responseHandlers.set(command, { resolve, reject });
    }
    _receive(message) {
        // Discard all messages from unwanted sources
        // or which are not replies
        // or which are not from the correct origin
        if (!message.data
            || !message.data.status
            || !message.data.id
            || (this._allowedOrigin !== '*' && message.origin !== this._allowedOrigin))
            return false;
        const data = message.data;
        const callback = this._getCallback(data.id);
        const state = this._waitingRequests.getState(data.id);
        if (callback) {
            if (!this._preserveRequests) {
                this._waitingRequests.remove(data.id);
                this._responseHandlers.delete(data.id);
            }
            console.debug('RpcClient RECEIVE', data);
            if (data.status === ResponseStatus.OK) {
                callback.resolve(data.result, data.id, state);
            }
            else if (data.status === ResponseStatus.ERROR) {
                const error = new Error(data.result.message);
                if (data.result.stack) {
                    error.stack = data.result.stack;
                }
                if (data.result.name) {
                    error.name = data.result.name;
                }
                callback.reject(error, data.id, state);
            }
            return true;
        }
        else {
            console.warn('Unknown RPC response:', data);
            return false;
        }
    }
    _getCallback(id) {
        // Response handlers by id have priority to more general ones by command
        if (this._responseHandlers.has(id)) {
            return this._responseHandlers.get(id);
        }
        else {
            const command = this._waitingRequests.getCommand(id);
            if (command) {
                return this._responseHandlers.get(command);
            }
        }
        return undefined;
    }
}
class PostMessageRpcClient extends RpcClient {
    constructor(targetWindow, allowedOrigin) {
        super(allowedOrigin);
        this._serverCloseCheckInterval = -1;
        this._target = targetWindow;
        this._connectionState = 0 /* DISCONNECTED */;
        this._receiveListener = this._receive.bind(this);
    }
    async init() {
        if (this._connectionState === 2 /* CONNECTED */) {
            return;
        }
        await this._connect();
        window.addEventListener('message', this._receiveListener);
        if (this._serverCloseCheckInterval !== -1)
            return;
        this._serverCloseCheckInterval = window.setInterval(() => this._checkIfServerClosed(), 300);
    }
    async call(command, ...args) {
        return this._call({
            command,
            args,
            id: RandomUtils.generateRandomId(),
        });
    }
    close() {
        // Clean up old requests and disconnect. Note that until the popup get's closed by the user
        // it's possible to connect again though by calling init.
        this._connectionState = 0 /* DISCONNECTED */;
        window.removeEventListener('message', this._receiveListener);
        window.clearInterval(this._serverCloseCheckInterval);
        this._serverCloseCheckInterval = -1;
        for (const [id, { reject }] of this._responseHandlers) {
            const state = this._waitingRequests.getState(id);
            reject('Connection was closed', typeof id === 'number' ? id : undefined, state);
        }
        this._waitingRequests.clear();
        this._responseHandlers.clear();
        if (this._target && this._target.closed)
            this._target = null;
    }
    _receive(message) {
        if (message.source !== this._target) {
            // ignore messages originating from another client's target window
            return false;
        }
        return super._receive(message);
    }
    async _call(request) {
        if (!this._target || this._target.closed) {
            throw new Error('Connection was closed.');
        }
        if (this._connectionState !== 2 /* CONNECTED */) {
            throw new Error('Client is not connected, call init first');
        }
        return new Promise((resolve, reject) => {
            // Store the request resolvers
            this._responseHandlers.set(request.id, { resolve, reject });
            this._waitingRequests.add(request.id, request.command);
            console.debug('RpcClient REQUEST', request.command, request.args);
            this._target.postMessage(request, this._allowedOrigin);
        });
    }
    _connect() {
        if (this._connectionState === 2 /* CONNECTED */)
            return;
        this._connectionState = 1 /* CONNECTING */;
        return new Promise((resolve, reject) => {
            const connectedListener = (message) => {
                const { source, origin, data } = message;
                if (source !== this._target
                    || data.status !== ResponseStatus.OK
                    || data.result !== 'pong'
                    || data.id !== 1
                    || (this._allowedOrigin !== '*' && origin !== this._allowedOrigin))
                    return;
                // Debugging printouts
                if (data.result.stack) {
                    const error = new Error(data.result.message);
                    error.stack = data.result.stack;
                    if (data.result.name) {
                        error.name = data.result.name;
                    }
                    console.error(error);
                }
                window.removeEventListener('message', connectedListener);
                this._connectionState = 2 /* CONNECTED */;
                console.log('RpcClient: Connection established');
                resolve(true);
            };
            window.addEventListener('message', connectedListener);
            /**
             * Send 'ping' command every 100ms, until cancelled
             */
            const tryToConnect = () => {
                if (this._connectionState === 2 /* CONNECTED */)
                    return;
                if (this._connectionState === 0 /* DISCONNECTED */
                    || this._checkIfServerClosed()) {
                    window.removeEventListener('message', connectedListener);
                    reject(new Error('Connection was closed'));
                    return;
                }
                try {
                    this._target.postMessage({ command: 'ping', id: 1 }, this._allowedOrigin);
                }
                catch (e) {
                    console.error(`postMessage failed: ${e}`);
                }
                window.setTimeout(tryToConnect, 100);
            };
            window.setTimeout(tryToConnect, 100);
        });
    }
    _checkIfServerClosed() {
        if (this._target && !this._target.closed)
            return false;
        this.close();
        return true;
    }
}
class RedirectRpcClient extends RpcClient {
    constructor(targetURL, allowedOrigin, preserveRequests = true) {
        super(allowedOrigin, /*storeState*/ true);
        this._target = targetURL;
        this._preserveRequests = preserveRequests;
    }
    async init() {
        // Check for a response in the URL (also removes params)
        const urlResponse = UrlRpcEncoder.receiveRedirectResponse(window.location);
        if (urlResponse) {
            this._receive(urlResponse);
            return;
        }
        // If there was no response in the URL it might be a history.back.
        if (this._rejectOnBack())
            return; // if rejectOnBack did reject do not look for a stored response
        // Check for a stored response referenced by a URL 'id' parameter
        const searchParams = new URLSearchParams(window.location.search);
        if (searchParams.has(UrlRpcEncoder.URL_SEARCHPARAM_NAME)) {
            const storedResponse = window.sessionStorage.getItem(`response-${searchParams.get(UrlRpcEncoder.URL_SEARCHPARAM_NAME)}`);
            if (storedResponse) {
                this._receive(JSONUtils.parse(storedResponse), false);
                return;
            }
        }
    }
    /* tslint:disable-next-line:no-empty */
    close() { }
    call(returnURL, command, handleHistoryBack = false, ...args) {
        this.callAndSaveLocalState(returnURL, null, command, handleHistoryBack, ...args);
    }
    callAndSaveLocalState(returnURL, state, command, handleHistoryBack = false, ...args) {
        const id = RandomUtils.generateRandomId();
        const url = UrlRpcEncoder.prepareRedirectInvocation(this._target, id, returnURL, command, args);
        this._waitingRequests.add(id, command, state);
        if (handleHistoryBack) {
            /**
             * The rpcBackRejectionId in the history.state is used to detect in the client
             * if a history entry was visited before, which makes it a history.back
             * navigation. The stored ID is then also used to retrieve the correct
             * stored callback and waiting request, to be able to reject it.
             */
            history.replaceState(Object.assign({}, history.state, { rpcBackRejectionId: id }), '');
        }
        console.debug('RpcClient REQUEST', command, args);
        window.location.href = url;
    }
    _receive(response, persistMessage = true) {
        const responseWasHandled = super._receive(response);
        if (responseWasHandled && persistMessage) {
            window.sessionStorage.setItem(`response-${response.data.id}`, JSONUtils.stringify(response));
        }
        return responseWasHandled;
    }
    _rejectOnBack() {
        if (!history.state || !history.state.rpcBackRejectionId)
            return false;
        const id = history.state.rpcBackRejectionId;
        // Delete the ID, so the request is not rejected again when the page is refreshed/revisited
        history.replaceState(Object.assign({}, history.state, { rpcBackRejectionId: null }), '');
        const callback = this._getCallback(id);
        const state = this._waitingRequests.getState(id);
        if (callback) {
            if (!this._preserveRequests) {
                this._waitingRequests.remove(id);
                this._responseHandlers.delete(id);
            }
            console.debug('RpcClient BACK');
            const error = new Error('Request aborted');
            callback.reject(error, id, state);
            return true;
        }
        return false;
    }
}

class RequestBehavior {
    constructor(type) {
        this._type = type;
    }
    static getAllowedOrigin(endpoint) {
        const url = new URL(endpoint);
        return url.origin;
    }
    async request(endpoint, command, args) {
        throw new Error('Not implemented');
    }
}
var BehaviorType;
(function (BehaviorType) {
    BehaviorType[BehaviorType["REDIRECT"] = 0] = "REDIRECT";
    BehaviorType[BehaviorType["POPUP"] = 1] = "POPUP";
    BehaviorType[BehaviorType["IFRAME"] = 2] = "IFRAME";
})(BehaviorType || (BehaviorType = {}));
class RedirectRequestBehavior extends RequestBehavior {
    static withLocalState(localState) {
        return new RedirectRequestBehavior(undefined, localState);
    }
    constructor(returnUrl, localState) {
        super(BehaviorType.REDIRECT);
        const location = window.location;
        this._returnUrl = returnUrl || `${location.origin}${location.pathname}`;
        this._localState = localState || {};
        // Reject local state with reserved property.
        if (typeof this._localState.__command !== 'undefined') {
            throw new Error('Invalid localState: Property \'__command\' is reserved');
        }
    }
    async request(endpoint, command, args) {
        const origin = RequestBehavior.getAllowedOrigin(endpoint);
        const client = new RedirectRpcClient(endpoint, origin);
        await client.init();
        const state = Object.assign({}, this._localState, { __command: command });
        client.callAndSaveLocalState(this._returnUrl, state, command, true, ...args);
    }
}
class PopupRequestBehavior extends RequestBehavior {
    constructor(options = PopupRequestBehavior.DEFAULT_OPTIONS) {
        super(BehaviorType.POPUP);
        this._options = options;
    }
    async request(endpoint, command, args) {
        const origin = RequestBehavior.getAllowedOrigin(endpoint);
        const popup = this.createPopup(endpoint);
        const client = new PostMessageRpcClient(popup, origin);
        await client.init();
        try {
            return await client.call(command, ...args);
        }
        catch (e) {
            throw e;
        }
        finally {
            client.close();
            popup.close();
        }
    }
    createPopup(url) {
        const popup = window.open(url, 'NimiqAccounts', this._options);
        if (!popup) {
            throw new Error('Failed to open popup');
        }
        return popup;
    }
}
PopupRequestBehavior.DEFAULT_OPTIONS = '';
class IFrameRequestBehavior extends RequestBehavior {
    constructor() {
        super(BehaviorType.IFRAME);
        this._iframe = null;
        this._client = null;
    }
    async request(endpoint, command, args) {
        if (this._iframe && this._iframe.src !== `${endpoint}${IFrameRequestBehavior.IFRAME_PATH_SUFFIX}`) {
            throw new Error('Hub iframe is already opened with another endpoint');
        }
        const origin = RequestBehavior.getAllowedOrigin(endpoint);
        if (!this._iframe) {
            this._iframe = await this.createIFrame(endpoint);
        }
        if (!this._iframe.contentWindow) {
            throw new Error(`IFrame contentWindow is ${typeof this._iframe.contentWindow}`);
        }
        if (!this._client) {
            this._client = new PostMessageRpcClient(this._iframe.contentWindow, origin);
            await this._client.init();
        }
        return await this._client.call(command, ...args);
    }
    async createIFrame(endpoint) {
        return new Promise((resolve, reject) => {
            const $iframe = document.createElement('iframe');
            $iframe.name = 'NimiqAccountsIFrame';
            $iframe.style.display = 'none';
            document.body.appendChild($iframe);
            $iframe.src = `${endpoint}${IFrameRequestBehavior.IFRAME_PATH_SUFFIX}`;
            $iframe.onload = () => resolve($iframe);
            $iframe.onerror = reject;
        });
    }
}
IFrameRequestBehavior.IFRAME_PATH_SUFFIX = '/iframe.html';

var RequestType;
(function (RequestType) {
    RequestType["LIST"] = "list";
    RequestType["MIGRATE"] = "migrate";
    RequestType["CHECKOUT"] = "checkout";
    RequestType["SIGN_MESSAGE"] = "sign-message";
    RequestType["SIGN_TRANSACTION"] = "sign-transaction";
    RequestType["ONBOARD"] = "onboard";
    RequestType["SIGNUP"] = "signup";
    RequestType["LOGIN"] = "login";
    RequestType["EXPORT"] = "export";
    RequestType["CHANGE_PASSWORD"] = "change-password";
    RequestType["LOGOUT"] = "logout";
    RequestType["ADD_ADDRESS"] = "add-address";
    RequestType["RENAME"] = "rename";
    RequestType["CHOOSE_ADDRESS"] = "choose-address";
})(RequestType || (RequestType = {}));

class HubApi {
    constructor(endpoint = HubApi.DEFAULT_ENDPOINT, defaultBehavior) {
        this._endpoint = endpoint;
        this._defaultBehavior = defaultBehavior || new PopupRequestBehavior(`left=${window.innerWidth / 2 - 400},top=75,width=800,height=850,location=yes,dependent=yes`);
        this._iframeBehavior = new IFrameRequestBehavior();
        // Check for RPC results in the URL
        this._redirectClient = new RedirectRpcClient('', RequestBehavior.getAllowedOrigin(this._endpoint));
    }
    static get DEFAULT_ENDPOINT() {
        const originArray = location.origin.split('.');
        originArray.shift();
        const tld = originArray.join('.');
        switch (tld) {
            case 'nimiq.com':
                return 'https://hub.nimiq.com';
            case 'nimiq-testnet.com':
                return 'https://hub.nimiq-testnet.com';
            default:
                return 'http://localhost:8080';
        }
    }
    checkRedirectResponse() {
        return this._redirectClient.init();
    }
    on(command, resolve, reject) {
        this._redirectClient.onResponse(command, 
        // State is always an object containing at least the __command property
        (result, rpcId, state) => resolve(result, state), (error, rpcId, state) => {
            if (!reject)
                return;
            reject(error, state);
        });
    }
    /**
     * Public API
     */
    checkout(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.CHECKOUT, [request]);
    }
    chooseAddress(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.CHOOSE_ADDRESS, [request]);
    }
    signTransaction(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.SIGN_TRANSACTION, [request]);
    }
    signMessage(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.SIGN_MESSAGE, [request]);
    }
    /**
     * Account Management
     *
     * Only accessible from Nimiq domains.
     */
    onboard(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.ONBOARD, [request]);
    }
    signup(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.SIGNUP, [request]);
    }
    login(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.LOGIN, [request]);
    }
    logout(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.LOGOUT, [request]);
    }
    export(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.EXPORT, [request]);
    }
    changePassword(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.CHANGE_PASSWORD, [request]);
    }
    addAddress(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.ADD_ADDRESS, [request]);
    }
    rename(request, requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.RENAME, [request]);
    }
    migrate(requestBehavior = this._defaultBehavior) {
        return this._request(requestBehavior, RequestType.MIGRATE, [{ appName: 'Account list' }]);
    }
    /**
     * Only accessible in iframe from Nimiq domains.
     */
    list(requestBehavior = this._iframeBehavior) {
        return this._request(requestBehavior, RequestType.LIST, []);
    }
    // END API
    /* PRIVATE METHODS */
    _request(behavior, command, args) {
        return behavior.request(this._endpoint, command, args);
    }
}
HubApi.RequestType = RequestType;
HubApi.RedirectRequestBehavior = RedirectRequestBehavior;
HubApi.MSG_PREFIX = '\x16Nimiq Signed Message:\n';

const HubUrl = {
    main: 'https://hub.nimiq.com',
    test: 'https://hub.nimiq-testnet.com',
};

const Hexagon = props => (core.h("svg", { viewBox: "0 0 27 24", class: props.class },
    core.h("path", { d: "M26.6991 10.875L21.0741 1.125C20.6691 0.4275 19.9266 0 19.1241 0H7.87414C7.07164 0 6.32914 0.4275 5.92789 1.125L0.302891 10.875C-0.0983594 11.5725 -0.0983594 12.4275 0.302891 13.125L5.92789 22.875C6.32914 23.5725 7.07164 24 7.87414 24H19.1241C19.9266 24 20.6691 23.5725 21.0704 22.875L26.6954 13.125C27.1004 12.4275 27.1004 11.5725 26.6991 10.875Z", fill: "currentColor" })));
const Close = props => (core.h("svg", { viewBox: "0 0 24 24", class: props.class },
    core.h("path", { "fill-rule": "evenodd", "clip-rule": "evenodd", d: "M3.528 3.52c4.683-4.684 12.275-4.686 16.96-.005 4.678 4.69 4.678 12.28 0 16.97-4.685 4.68-12.277 4.678-16.96-.005-4.682-4.684-4.682-12.276 0-16.96zm13.145 13.133a1 1 0 0 0 .036-1.374l-3.11-3.11a.25.25 0 0 1 0-.352l3.11-3.11a1 1 0 1 0-1.414-1.415l-3.11 3.11a.25.25 0 0 1-.354 0l-3.11-3.11a1 1 0 0 0-1.41 1.415l3.11 3.11a.249.249 0 0 1 0 .353l-3.11 3.109a1 1 0 0 0 0 1.415c.396.38 1.021.38 1.416 0l3.109-3.11a.252.252 0 0 1 .354 0l3.11 3.11a1 1 0 0 0 1.373-.041z", fill: "currentColor" })));

const Checkout = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        this.inProgress = false;
        /**
         * The theme of the button. Use light when using against dark background.
         */
        this.theme = 'neutral';
        /**
         * The network you want to use. Can be 'main' or 'test'.
         */
        this.network = 'main';
        /**
         * The text to display on the button. Default: "Pay with NIM"
         */
        this.text = 'Pay with NIM';
        this.nimCheckoutSuccess = core.createEvent(this, "nimCheckoutSuccess", 7);
        this.nimCheckoutError = core.createEvent(this, "nimCheckoutError", 7);
    }
    async checkout() {
        this.inProgress = true;
        let options = {
            appName: this.app,
            recipient: this.recipient,
            value: this.value,
        };
        if (this.logoUrl) {
            options.shopLogoUrl = this.logoUrl;
        }
        if (this.fee) {
            options.fee = this.fee;
        }
        if (this.data) {
            options.extraData = this.data;
        }
        const hubApi = new HubApi(HubUrl[this.network]);
        try {
            const transaction = await hubApi.checkout(options);
            this.nimCheckoutSuccess.emit(transaction);
            this.inProgress = false;
        }
        catch (error) {
            this.nimCheckoutError.emit(error);
            this.inProgress = false;
        }
    }
    render() {
        return (core.h("button", { class: 'nim-theme-' + this.theme, disabled: this.inProgress, type: "button", onClick: _ => this.checkout() }, core.h(Hexagon, { class: "nim-icon nim-logo" }), core.h("div", null, this.inProgress ? core.h("span", null, "In progress...") : core.h("span", null, this.text))));
    }
    static get style() { return ":host{display:inline-block;--nimiq-blue:#1f2348;--nimiq-light-blue:#0582ca;--nimiq-gold:#e9b213;--nimiq-green:#21bca5;--nimiq-orange:#fc8702;--nimiq-red:#d94432;--nimiq-gray:#e2e8f0;--nimiq-light-gray:#edf2f7;--nimiq-blue-darkened:#151833;--nimiq-light-blue-darkened:#0071c3;--nimiq-gold-darkened:#e5a212;--nimiq-green-darkened:#20b29e;--nimiq-orange-darkened:#fc7500;--nimiq-red-darkened:#d13030;--nimiq-blue-bg:radial-gradient(ellipse at bottom right,#260133,var(--nimiq-blue));--nimiq-light-blue-bg:radial-gradient(ellipse at bottom right,#265dd7,var(--nimiq-light-blue));--nimiq-gold-bg:radial-gradient(ellipse at bottom right,#ec991c,var(--nimiq-gold));--nimiq-green-bg:radial-gradient(ellipse at bottom right,#41a38e,var(--nimiq-green));--nimiq-orange-bg:radial-gradient(ellipse at bottom right,#fd6216,var(--nimiq-orange));--nimiq-red-bg:radial-gradient(ellipse at bottom right,#cc3047,var(--nimiq-red));--nimiq-blue-bg-darkened:radial-gradient(ellipse at bottom right,#180021,var(--nimiq-blue-darkened));--nimiq-light-blue-bg-darkened:radial-gradient(ellipse at bottom right,#2355c4,var(--nimiq-light-blue-darkened));--nimiq-gold-bg-darkened:radial-gradient(ellipse at bottom right,#e58a1b,var(--nimiq-gold-darkened));--nimiq-green-bg-darkened:radial-gradient(ellipse at bottom right,#3d9988,var(--nimiq-green-darkened));--nimiq-orange-bg-darkened:radial-gradient(ellipse at bottom right,#ea5200,var(--nimiq-orange-darkened));--nimiq-red-bg-darkened:radial-gradient(ellipse at bottom right,#bf2d46,var(--nimiq-red-darkened))}button{border-radius:100px;border:0 none;padding:12px 20px;margin:0;display:-ms-flexbox;display:flex;-ms-flex-direction:row;flex-direction:row;-ms-flex-align:center;align-items:center;font-size:16px;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Open Sans,Helvetica Neue,sans-serif;cursor:pointer;position:relative;-webkit-box-shadow:none;box-shadow:none;-webkit-transition:all .3s cubic-bezier(.25,0,0,1);transition:all .3s cubic-bezier(.25,0,0,1)}button:after{content:\"\";position:absolute;left:-5px;top:-5px;right:-5px;bottom:-5px;border:2px solid rgba(5,130,202,.5);border-radius:500px;opacity:0}button:focus{outline:none}button:focus:not([disabled]):after{opacity:1}button:hover{-webkit-transform:translateY(-2px);transform:translateY(-2px);-webkit-box-shadow:0 .6rem 1rem rgba(0,0,0,.1);box-shadow:0 .6rem 1rem rgba(0,0,0,.1)}button[disabled]{background:var(--nimiq-gray)!important;-webkit-box-shadow:none!important;box-shadow:none!important;-webkit-transform:none!important;transform:none!important;color:var(--nimiq-blue)!important;cursor:not-allowed;opacity:.4}button[disabled]>.nim-icon{color:var(--nimiq-blue)}.nim-logo{width:24px;height:24px;margin-right:12px}.nim-theme-neutral{color:var(--nimiq-blue);background-color:var(--nimiq-light-gray)}.nim-theme-neutral:hover{background-color:var(--nimiq-gray);-webkit-box-shadow:none;box-shadow:none}.nim-theme-neutral>.nim-icon{color:#e9b213}.nim-theme-blue{color:#fff;background-image:var(--nimiq-blue-bg)}.nim-theme-blue:hover{color:#fff;background-image:var(--nimiq-blue-bg-darkened)}.nim-theme-blue>.nim-icon{color:var(--nimiq-gold)}.nim-theme-gold{color:#fff;background-image:var(--nimiq-gold-bg)}.nim-theme-gold:hover{color:#fff;background-image:var(--nimiq-gold-bg-darkened)}.nim-theme-gold>.nim-icon{color:#fff}.nim-theme-light-blue{color:#fff;background-image:var(--nimiq-light-blue-bg)}.nim-theme-light-blue:hover{color:#fff;background-image:var(--nimiq-light-blue-bg-darkened)}.nim-theme-light-blue>.nim-icon{color:#fff}.nim-theme-green{color:#fff;background-image:var(--nimiq-green-bg)}.nim-theme-green:hover{color:#fff;background-image:var(--nimiq-green-bg-darkened)}.nim-theme-green>.nim-icon{color:#fff}.nim-theme-orange{color:#fff;background-image:var(--nimiq-orange-bg)}.nim-theme-orange:hover{color:#fff;background-image:var(--nimiq-orange-bg-darkened)}.nim-theme-orange>.nim-icon{color:#fff}.nim-theme-red{color:#fff;background-image:var(--nimiq-red-bg)}.nim-theme-red:hover{color:#fff;background-image:var(--nimiq-red-bg-darkened)}.nim-theme-red>.nim-icon{color:#fff}"; }
};

class Iqons{static async svg(t){const e=makeHash(t);return this._svgTemplate(e[0],e[2],e[3]+e[4],e[5]+e[6],e[7]+e[8],e[9]+e[10],e[11])}static async render(t,e){e.innerHTML=await this.svg(t);}static async toDataUrl(t){return `data:image/svg+xml;base64,${this._btoa(await this.svg(t,!0))}`}static placeholder(t="#bbb",e=1){return `<svg viewBox="0 0 160 160" width="160" height="160" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/2000/xlink" >\n<path fill="none" stroke="${t}" stroke-width="${2*e}" transform="translate(0, 8) scale(0.5)" d="M251.6 17.34l63.53 110.03c5.72 9.9 5.72 22.1 0 32L251.6 269.4c-5.7 9.9-16.27 16-27.7 16H96.83c-11.43 0-22-6.1-27.7-16L5.6 159.37c-5.7-9.9-5.7-22.1 0-32L69.14 17.34c5.72-9.9 16.28-16 27.7-16H223.9c11.43 0 22 6.1 27.7 16z"/>\n<g transform="scale(0.9) translate(9, 8)">\n<circle cx="80" cy="80" r="40" fill="none" stroke="${t}" stroke-width="${e}" opacity=".9"></circle>\n<g opacity=".1" fill="#010101"><path d="M119.21,80a39.46,39.46,0,0,1-67.13,28.13c10.36,2.33,36,3,49.82-14.28,10.39-12.47,8.31-33.23,4.16-43.26A39.35,39.35,0,0,1,119.21,80Z"/></g>\n</g></svg>`}static renderPlaceholder(t,e,s){t.innerHTML=this.placeholder(e,s);}static placeholderToDataUrl(t,e){return `data:image/svg+xml;base64,${this._btoa(this.placeholder(t,e))}`}static async image(t){const e=await this.toDataUrl(t),s=await this._loadImage(e);return s.style.width="100%",s.style.height="100%",s}static async _svgTemplate(t,e,s,a,n,i,r){return this._$svg(await this._$iqons(t,e,s,a,n,i,r))}static async _$iqons(t,e,s,a,n,i,r){const o=hashToRGB(t,e,r);return t=o.main,e=o.background,`<g color="${t}" fill="${r=o.accent}">\n<rect fill="${e}" x="0" y="0" width="160" height="160"></rect>\n<circle cx="80" cy="80" r="40" fill="${t}"></circle>\n<g opacity=".1" fill="#010101"><path d="M119.21,80a39.46,39.46,0,0,1-67.13,28.13c10.36,2.33,36,3,49.82-14.28,10.39-12.47,8.31-33.23,4.16-43.26A39.35,39.35,0,0,1,119.21,80Z"/></g>\n${await this._generatePart("top",a)}\n${await this._generatePart("side",n)}\n${await this._generatePart("face",s)}\n${await this._generatePart("bottom",i)}\n</g>`}static _$svg(t){const e=this._getRandomId();return `<svg viewBox="0 0 160 160" width="160" height="160" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/2000/xlink" >\n<defs><clipPath id="hexagon-clip-${e}">\n<path d="M251.6 17.34l63.53 110.03c5.72 9.9 5.72 22.1 0 32L251.6 269.4c-5.7 9.9-16.27 16-27.7 16H96.83c-11.43 0-22-6.1-27.7-16L5.6 159.37c-5.7-9.9-5.7-22.1 0-32L69.14 17.34c5.72-9.9 16.28-16 27.7-16H223.9c11.43 0 22 6.1 27.7 16z" transform="scale(0.5) translate(0, 16)"/>\n</clipPath></defs>\n<g clip-path="url(#hexagon-clip-${e})">\n${t}\n</g></svg>`}static async _generatePart(t,e){const s=await this._getAssets(),a=t+"_"+this._assetIndex(e,t),n=s.getElementById(a);return n?n.innerHTML:""}static _loadImage(t){return new Promise((e,s)=>{const a=document.createElement("img");a.addEventListener("load",t=>e(a),{once:!0}),a.src=t;})}static async _getAssets(){return this._assetsPromise||(this._assetsPromise=new Promise(async function(t){let e;e="undefined"!=typeof IqonsAssets?IqonsAssets:await fetch(self.NIMIQ_IQONS_SVG_PATH||Iqons.svgPath).then(t=>t.text()),"undefined"!=typeof module&&module.exports&&(global.DOMParser=require("dom-parser")),t((new DOMParser).parseFromString(e,"image/svg+xml"));}))}static _btoa(t){return "undefined"!=typeof module&&module.exports?new Buffer(t).toString("base64"):btoa(t)}static _assetIndex(t,e){return (t=Number(t)%Iqons.ASSET_COUNTS[e]+1)<10&&(t="0"+t),t}static _getRandomId(){return Math.floor(256*Math.random())}}
// @asset(/node_modules/@nimiq/iqons/dist/iqons.min.svg);
Iqons.svgPath="/node_modules/@nimiq/iqons/dist/iqons.min.svg",Iqons.ASSET_COUNTS={top:21,side:21,face:21,bottom:21};
function makeHash(t){const r=(""+t.split("").map(t=>Number(t.charCodeAt(0))+3).reduce((t,r)=>t*(1-t)*__chaosHash(r),.5)).split("").reduce((t,r)=>r+t,"");return _padEnd(r.replace(".",r[5]).substr(4,17),13,r[5])}function __chaosHash(t){let r=1/t;for(let t=0;t<100;t++)r=(1-r)*r*3.569956786876;return r}function _padEnd(t,r,e){if(String.prototype.padEnd)return t.padEnd(r,e);for(;t.length<r;)t+=e;return t.substring(0,Math.max(t.length,r))}
function hashToRGB(o,n,r){return indicesToRGB(hashToIndices(o,n,r))}function hashToIndices(o,n,r){for(o=parseInt(o,10),n=parseInt(n,10),r=parseInt(r,10),o===n&&++o>9&&(o=0);r===o||r===n;)++r>9&&(r=0);return {main:o,background:n,accent:r}}function indicesToRGB(o){return {main:colors[o.main],background:backgroundColors[o.background],accent:colors[o.accent]}}const colors=["#FC8702","#D94432","#E9B213","#1A5493","#0582CA","#5961A8","#21BCA5","#FA7268","#88B04B","#795548"];const backgroundColors=["#FC8702","#D94432","#E9B213","#1F2348","#0582CA","#5F4B8B","#21BCA5","#FA7268","#88B04B","#795548"];

var Status;
(function (Status) {
    Status[Status["Idle"] = 0] = "Idle";
    Status[Status["Progress"] = 1] = "Progress";
    Status[Status["Success"] = 2] = "Success";
    Status[Status["Error"] = 3] = "Error";
})(Status || (Status = {}));
const Donate = class {
    constructor(hostRef) {
        core.registerInstance(this, hostRef);
        this.status = Status.Idle;
        this.isOpen = false;
        this.addressChunks = [];
        /**
         * The theme of the button. Use light when using against dark background.
         */
        this.theme = 'neutral';
        /**
         * The network you want to use. Can be 'main' or 'test'.
         */
        this.network = 'main';
        /**
         * The text to display on the button. Default: "Donate NIM"
         */
        this.text = 'Donate NIM';
        /**
         * The text to display once payment is done. Default: "Thank you for the donation!"
         */
        this.thankyou = 'Thank you for the donation!';
    }
    onRecipientChanged() {
        this.breakAddressIntoChunks();
        this.generateIdenticon();
    }
    componentWillLoad() {
        Iqons.svgPath = core.getAssetPath('./identicons/iqons.min.svg');
        this.breakAddressIntoChunks();
        this.generateIdenticon();
    }
    breakAddressIntoChunks() {
        const addressWithoutSpaces = this.recipient.replace(/\s/g, '');
        this.addressChunks = addressWithoutSpaces.match(/.{1,4}/g);
    }
    generateIdenticon() {
        Iqons.toDataUrl(this.recipient).then(identicon => {
            this.identicon = identicon;
        });
    }
    open() {
        this.isOpen = true;
    }
    close() {
        this.isOpen = false;
    }
    async donate(event) {
        event.preventDefault();
        this.status = Status.Progress;
        let options = {
            appName: this.text,
            recipient: this.recipient,
            value: this.amount * 100000,
        };
        if (this.logoUrl) {
            options.shopLogoUrl = this.logoUrl;
        }
        if (this.fee) {
            options.fee = this.fee;
        }
        if (this.message) {
            options.extraData = this.message.trim();
        }
        const hubApi = new HubApi(HubUrl[this.network]);
        try {
            await hubApi.checkout(options);
            this.status = Status.Success;
            setTimeout(() => {
                this.status = Status.Idle;
                this.close();
            }, 2000);
        }
        catch (error) {
            this.status = Status.Error;
            setTimeout(() => {
                this.status = Status.Idle;
            }, 2000);
        }
    }
    updateAmount(event) {
        this.amount = parseFloat(event.target.value);
    }
    updateMessage(event) {
        this.message = event.target.value;
    }
    canConfirm() {
        return this.amount > 0;
    }
    render() {
        return (core.h("div", { class: "donate" }, core.h("button", { class: 'nim-button nim-theme-' + this.theme, disabled: this.status === Status.Progress, type: "button", onClick: _ => this.open() }, core.h(Hexagon, { class: "nim-icon nim-logo" }), core.h("div", null, core.h("span", null, this.text))), core.h("div", { class: { overlay: true, 'is-open': this.isOpen } }, core.h("div", { class: "dialog" }, core.h("button", { onClick: _ => this.close(), type: "button", class: "close" }, core.h(Close, { class: "nim-icon" })), core.h("form", { onSubmit: e => this.donate(e) }, core.h("h2", null, "Donate"), core.h("div", { class: "recipient" }, core.h("div", { class: "identicon" }, core.h("img", { src: this.identicon })), core.h("div", { class: "address" }, this.addressChunks.map(chunk => (core.h("span", { class: "chunk" }, chunk))))), core.h("div", { class: "amount" }, core.h("input", { onInput: e => this.updateAmount(e), type: "text", placeholder: "0.00" }), core.h("span", { class: "ticker" }, "NIM")), core.h("div", { class: "message" }, core.h("input", { onInput: e => this.updateMessage(e), type: "text", maxLength: 64, placeholder: "Add a message..." })), core.h("div", { class: "confirm" }, core.h("button", { disabled: !this.canConfirm(), class: "nim-button nim-theme-light-blue", type: "submit" }, "Confirm"))), core.h("div", { class: {
                flash: true,
                show: this.status !== Status.Idle,
            } }, core.h("p", null, "Transaction in progress...")), core.h("div", { class: {
                'flash success': true,
                show: this.status === Status.Success,
            } }, core.h("p", null, this.thankyou)), core.h("div", { class: {
                'flash error': true,
                show: this.status === Status.Error,
            } }, core.h("p", null, "Transaction failed. Please try again."))))));
    }
    static get assetsDirs() { return ["identicons"]; }
    static get watchers() { return {
        "recipient": ["onRecipientChanged"]
    }; }
    static get style() { return ":host{display:inline-block;--nimiq-blue:#1f2348;--nimiq-light-blue:#0582ca;--nimiq-gold:#e9b213;--nimiq-green:#21bca5;--nimiq-orange:#fc8702;--nimiq-red:#d94432;--nimiq-gray:#e2e8f0;--nimiq-light-gray:#edf2f7;--nimiq-blue-darkened:#151833;--nimiq-light-blue-darkened:#0071c3;--nimiq-gold-darkened:#e5a212;--nimiq-green-darkened:#20b29e;--nimiq-orange-darkened:#fc7500;--nimiq-red-darkened:#d13030;--nimiq-blue-bg:radial-gradient(ellipse at bottom right,#260133,var(--nimiq-blue));--nimiq-light-blue-bg:radial-gradient(ellipse at bottom right,#265dd7,var(--nimiq-light-blue));--nimiq-gold-bg:radial-gradient(ellipse at bottom right,#ec991c,var(--nimiq-gold));--nimiq-green-bg:radial-gradient(ellipse at bottom right,#41a38e,var(--nimiq-green));--nimiq-orange-bg:radial-gradient(ellipse at bottom right,#fd6216,var(--nimiq-orange));--nimiq-red-bg:radial-gradient(ellipse at bottom right,#cc3047,var(--nimiq-red));--nimiq-blue-bg-darkened:radial-gradient(ellipse at bottom right,#180021,var(--nimiq-blue-darkened));--nimiq-light-blue-bg-darkened:radial-gradient(ellipse at bottom right,#2355c4,var(--nimiq-light-blue-darkened));--nimiq-gold-bg-darkened:radial-gradient(ellipse at bottom right,#e58a1b,var(--nimiq-gold-darkened));--nimiq-green-bg-darkened:radial-gradient(ellipse at bottom right,#3d9988,var(--nimiq-green-darkened));--nimiq-orange-bg-darkened:radial-gradient(ellipse at bottom right,#ea5200,var(--nimiq-orange-darkened));--nimiq-red-bg-darkened:radial-gradient(ellipse at bottom right,#bf2d46,var(--nimiq-red-darkened))}.donate{font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Open Sans,Helvetica Neue,sans-serif}button{font-size:16px;border:0 none;background:none;cursor:pointer}.nim-button{border-radius:100px;padding:12px 20px;margin:0;display:-ms-flexbox;display:flex;-ms-flex-direction:row;flex-direction:row;-ms-flex-align:center;align-items:center;position:relative;-webkit-box-shadow:none;box-shadow:none;-webkit-transition:all .3s cubic-bezier(.25,0,0,1);transition:all .3s cubic-bezier(.25,0,0,1)}button:after{content:\"\";position:absolute;left:-5px;top:-5px;right:-5px;bottom:-5px;border:2px solid rgba(5,130,202,.5);border-radius:500px;opacity:0}button:focus{outline:none}button:focus:not([disabled]):after{opacity:1}.nim-button:hover{-webkit-transform:translateY(-2px);transform:translateY(-2px);-webkit-box-shadow:0 .6rem 1rem rgba(0,0,0,.1);box-shadow:0 .6rem 1rem rgba(0,0,0,.1)}.nim-button[disabled]{background:var(--nimiq-gray)!important;-webkit-box-shadow:none!important;box-shadow:none!important;-webkit-transform:none!important;transform:none!important;color:var(--nimiq-blue)!important;cursor:not-allowed;opacity:.4}.nim-button[disabled]>.nim-icon{color:var(--nimiq-blue)}.nim-logo{width:24px;height:24px;margin-right:12px}.nim-theme-neutral{color:var(--nimiq-blue);background-color:var(--nimiq-light-gray)}.nim-theme-neutral:hover{background-color:var(--nimiq-gray);-webkit-box-shadow:none;box-shadow:none}.nim-theme-neutral>.nim-icon{color:#e9b213}.nim-theme-blue{color:#fff;background-image:var(--nimiq-blue-bg)}.nim-theme-blue:hover{color:#fff;background-image:var(--nimiq-blue-bg-darkened)}.nim-theme-blue>.nim-icon{color:var(--nimiq-gold)}.nim-theme-gold{color:#fff;background-image:var(--nimiq-gold-bg)}.nim-theme-gold:hover{color:#fff;background-image:var(--nimiq-gold-bg-darkened)}.nim-theme-gold>.nim-icon{color:#fff}.nim-theme-light-blue{color:#fff;background-image:var(--nimiq-light-blue-bg)}.nim-theme-light-blue:hover{color:#fff;background-image:var(--nimiq-light-blue-bg-darkened)}.nim-theme-light-blue>.nim-icon{color:#fff}.nim-theme-green{color:#fff;background-image:var(--nimiq-green-bg)}.nim-theme-green:hover{color:#fff;background-image:var(--nimiq-green-bg-darkened)}.nim-theme-green>.nim-icon{color:#fff}.nim-theme-orange{color:#fff;background-image:var(--nimiq-orange-bg)}.nim-theme-orange:hover{color:#fff;background-image:var(--nimiq-orange-bg-darkened)}.nim-theme-orange>.nim-icon{color:#fff}.nim-theme-red{color:#fff;background-image:var(--nimiq-red-bg)}.nim-theme-red:hover{color:#fff;background-image:var(--nimiq-red-bg-darkened)}.nim-theme-red>.nim-icon{color:#fff}.overlay{width:0;height:0;overflow:hidden;display:-ms-flexbox;display:flex;position:fixed;top:0;right:0;bottom:0;left:0;background-color:rgba(31,35,72,.8);-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;z-index:-1}.overlay.is-open{width:100%;height:100%;z-index:500}.dialog{position:relative;max-width:320px;width:90%;color:var(--nimiq-blue);background-color:#fff;-webkit-box-shadow:0 .5rem 3.5rem rgba(0,0,0,.12);box-shadow:0 .5rem 3.5rem rgba(0,0,0,.12);border-radius:8px;-webkit-transform:translateY(24px);transform:translateY(24px);opacity:0;-webkit-transition:opacity .4s cubic-bezier(.25,0,0,1),-webkit-transform .4s cubic-bezier(.25,0,0,1);transition:opacity .4s cubic-bezier(.25,0,0,1),-webkit-transform .4s cubic-bezier(.25,0,0,1);transition:transform .4s cubic-bezier(.25,0,0,1),opacity .4s cubic-bezier(.25,0,0,1);transition:transform .4s cubic-bezier(.25,0,0,1),opacity .4s cubic-bezier(.25,0,0,1),-webkit-transform .4s cubic-bezier(.25,0,0,1)}.overlay.is-open>.dialog{-webkit-transform:translateY(0);transform:translateY(0);opacity:1}.flash{position:absolute;top:0;right:0;bottom:0;left:0;background-image:var(--nimiq-blue-bg);color:#fff;border-radius:4px;margin:8px;display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;font-weight:200;opacity:0;pointer-events:none}.flash.show{opacity:1;pointer-events:all;-webkit-transition:opacity .3s cubic-bezier(.25,0,0,1);transition:opacity .3s cubic-bezier(.25,0,0,1)}.flash.error{background-image:var(--nimiq-red-bg)}.flash.success{background-image:var(--nimiq-green-bg)}.flash p{font-size:24px;text-align:center;margin:0;opacity:0;-webkit-transform:translateY(24px);transform:translateY(24px)}.flash.show p{padding:24px;opacity:1;-webkit-transform:translateY(0);transform:translateY(0);-webkit-transition:opacity .3s cubic-bezier(.25,0,0,1),-webkit-transform .3s cubic-bezier(.25,0,0,1) .1s;transition:opacity .3s cubic-bezier(.25,0,0,1),-webkit-transform .3s cubic-bezier(.25,0,0,1) .1s;transition:opacity .3s cubic-bezier(.25,0,0,1),transform .3s cubic-bezier(.25,0,0,1) .1s;transition:opacity .3s cubic-bezier(.25,0,0,1),transform .3s cubic-bezier(.25,0,0,1) .1s,-webkit-transform .3s cubic-bezier(.25,0,0,1) .1s}.dialog h2{text-align:center;font-size:24px;line-height:1.2;font-weight:600;padding:32px;margin:0}.close{position:absolute;top:0;right:0;width:24px;height:24px;margin:12px;padding:0;color:var(--nimiq-blue);opacity:.5}.close:hover{opacity:1}.close .nim-icon{display:inline}.recipient{display:-ms-flexbox;display:flex;margin-left:24px;margin-right:24px;padding-bottom:24px;border-bottom:solid 1px var(--nimiq-light-gray)}.recipient .identicon{width:72px;height:72px;margin-right:16px}.recipient .identicon img{width:100%;height:100%}.recipient .address{-ms-flex:1;flex:1;display:-ms-flexbox;display:flex;-ms-flex-wrap:wrap;flex-wrap:wrap;color:var(--nimiq-blue);opacity:.5;letter-spacing:.02em}.recipient .address>.chunk{display:block;width:33.3%}.confirm{margin-top:16px;padding:0 24px 24px}.confirm button{display:block;padding:16px;width:100%}.amount{margin-top:24px;font-size:24px;position:relative;margin-left:24px;margin-right:24px}.amount,.amount input{-webkit-box-sizing:border-box;box-sizing:border-box}.amount input{display:block;width:100%;font-family:inherit;font-size:inherit;padding:12px;border:2px solid rgba(31,35,72,.1);color:var(--nimiq-blue);background:0 0;border-radius:8px;outline:0;-webkit-transition:color .2s ease,border .2s ease;transition:color .2s ease,border .2s ease;background-clip:padding-box}.amount input:focus{border-color:rgba(5,130,202,.2)}.amount .ticker{display:block;position:absolute;right:12px;bottom:20px;margin:0;padding:0;font-size:16px;font-weight:600;letter-spacing:.08em;opacity:.4;-webkit-transition:opacity .2s ease-out;transition:opacity .2s ease-out;color:var(--nimiq-light-blue)}.message{margin-top:24px;padding-left:32px;padding-right:32px}.message input{display:block;width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;font-family:inherit;font-size:inherit;padding:12px 0;border:0 none;border-bottom:2px solid rgba(31,35,72,0);color:var(--nimiq-blue);background:0 0;outline:0;-webkit-transition:color .2s ease,border .2s ease;transition:color .2s ease,border .2s ease;background-clip:padding-box;text-align:center}.message input:hover{border-color:rgba(31,35,72,.1)}.message input:focus{border-color:rgba(5,130,202,.2)}"; }
};

exports.nim_checkout = Checkout;
exports.nim_donate = Donate;
