import { h } from "@stencil/core";
import HubApi from '@nimiq/hub-api';
import { HubUrl } from '../../utils/common';
import { Hexagon } from '../../utils/icons';
export class Checkout {
    constructor() {
        this.inProgress = false;
        /**
         * The theme of the button. Use light when using against dark background.
         */
        this.theme = 'neutral';
        /**
         * The network you want to use. Can be 'main' or 'test'.
         */
        this.network = 'main';
        /**
         * The text to display on the button. Default: "Pay with NIM"
         */
        this.text = 'Pay with NIM';
    }
    async checkout() {
        this.inProgress = true;
        let options = {
            appName: this.app,
            recipient: this.recipient,
            value: this.value,
        };
        if (this.logoUrl) {
            options.shopLogoUrl = this.logoUrl;
        }
        if (this.fee) {
            options.fee = this.fee;
        }
        if (this.data) {
            options.extraData = this.data;
        }
        const hubApi = new HubApi(HubUrl[this.network]);
        try {
            const transaction = await hubApi.checkout(options);
            this.nimCheckoutSuccess.emit(transaction);
            this.inProgress = false;
        }
        catch (error) {
            this.nimCheckoutError.emit(error);
            this.inProgress = false;
        }
    }
    render() {
        return (h("button", { class: 'nim-theme-' + this.theme, disabled: this.inProgress, type: "button", onClick: _ => this.checkout() },
            h(Hexagon, { class: "nim-icon nim-logo" }),
            h("div", null, this.inProgress ? h("span", null, "In progress...") : h("span", null, this.text))));
    }
    static get is() { return "nim-checkout"; }
    static get encapsulation() { return "shadow"; }
    static get originalStyleUrls() { return {
        "$": ["checkout.css"]
    }; }
    static get styleUrls() { return {
        "$": ["checkout.css"]
    }; }
    static get properties() { return {
        "theme": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "Theme",
                "resolved": "\"blue\" | \"gold\" | \"green\" | \"light-blue\" | \"neutral\" | \"orange\" | \"red\"",
                "references": {
                    "Theme": {
                        "location": "import",
                        "path": "../../utils/common"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The theme of the button. Use light when using against dark background."
            },
            "attribute": "theme",
            "reflect": false,
            "defaultValue": "'neutral'"
        },
        "network": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "Network",
                "resolved": "\"main\" | \"test\"",
                "references": {
                    "Network": {
                        "location": "import",
                        "path": "../../utils/common"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The network you want to use. Can be 'main' or 'test'."
            },
            "attribute": "network",
            "reflect": false,
            "defaultValue": "'main'"
        },
        "app": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The name of your app, should be as short as possible."
            },
            "attribute": "app",
            "reflect": false
        },
        "recipient": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The human-readable address of the recipient (your shop/app)."
            },
            "attribute": "recipient",
            "reflect": false
        },
        "value": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Value of the transaction, in Luna. 1 NIM = 100000 Luna."
            },
            "attribute": "value",
            "reflect": false
        },
        "logoUrl": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "An image URL. Must be on the same origin as the request is sent from. Should be square and at least 146x146 px."
            },
            "attribute": "logo-url",
            "reflect": false
        },
        "fee": {
            "type": "number",
            "mutable": false,
            "complexType": {
                "original": "number",
                "resolved": "number",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Transaction fee in luna. Default: 0"
            },
            "attribute": "fee",
            "reflect": false
        },
        "data": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Extra data that should be sent with the transaction."
            },
            "attribute": "data",
            "reflect": false
        },
        "text": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The text to display on the button. Default: \"Pay with NIM\""
            },
            "attribute": "text",
            "reflect": false,
            "defaultValue": "'Pay with NIM'"
        }
    }; }
    static get states() { return {
        "inProgress": {}
    }; }
    static get events() { return [{
            "method": "nimCheckoutSuccess",
            "name": "nimCheckoutSuccess",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when checkout is successful"
            },
            "complexType": {
                "original": "SignedTransaction",
                "resolved": "SignedTransaction",
                "references": {
                    "SignedTransaction": {
                        "location": "import",
                        "path": "../../utils/common"
                    }
                }
            }
        }, {
            "method": "nimCheckoutError",
            "name": "nimCheckoutError",
            "bubbles": true,
            "cancelable": true,
            "composed": true,
            "docs": {
                "tags": [],
                "text": "Emitted when an error happened during checkout or it is canceled"
            },
            "complexType": {
                "original": "any",
                "resolved": "any",
                "references": {}
            }
        }]; }
}
