import { r as registerInstance, c as createEvent, h } from './core-c9c72349.js';
import { H as HubApi, a as HubUrl, b as Hexagon } from './icons-6bc86ba7.js';

const Checkout = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.inProgress = false;
        /**
         * The theme of the button. Use light when using against dark background.
         */
        this.theme = 'neutral';
        /**
         * The network you want to use. Can be 'main' or 'test'.
         */
        this.network = 'main';
        /**
         * The text to display on the button. Default: "Pay with NIM"
         */
        this.text = 'Pay with NIM';
        this.nimCheckoutSuccess = createEvent(this, "nimCheckoutSuccess", 7);
        this.nimCheckoutError = createEvent(this, "nimCheckoutError", 7);
    }
    async checkout() {
        this.inProgress = true;
        let options = {
            appName: this.app,
            recipient: this.recipient,
            value: this.value,
        };
        if (this.logoUrl) {
            options.shopLogoUrl = this.logoUrl;
        }
        if (this.fee) {
            options.fee = this.fee;
        }
        if (this.data) {
            options.extraData = this.data;
        }
        const hubApi = new HubApi(HubUrl[this.network]);
        try {
            const transaction = await hubApi.checkout(options);
            this.nimCheckoutSuccess.emit(transaction);
            this.inProgress = false;
        }
        catch (error) {
            this.nimCheckoutError.emit(error);
            this.inProgress = false;
        }
    }
    render() {
        return (h("button", { class: 'nim-theme-' + this.theme, disabled: this.inProgress, type: "button", onClick: _ => this.checkout() }, h(Hexagon, { class: "nim-icon nim-logo" }), h("div", null, this.inProgress ? h("span", null, "In progress...") : h("span", null, this.text))));
    }
    static get style() { return ":host {\n  display: inline-block;\n  --nimiq-blue: #1f2348;\n  --nimiq-light-blue: #0582ca;\n  --nimiq-gold: #e9b213;\n  --nimiq-green: #21bca5;\n  --nimiq-orange: #fc8702;\n  --nimiq-red: #d94432;\n  --nimiq-gray: #e2e8f0;\n  --nimiq-light-gray: #edf2f7;\n  --nimiq-blue-darkened: #151833;\n  --nimiq-light-blue-darkened: #0071c3;\n  --nimiq-gold-darkened: #e5a212;\n  --nimiq-green-darkened: #20b29e;\n  --nimiq-orange-darkened: #fc7500;\n  --nimiq-red-darkened: #d13030;\n  --nimiq-blue-bg: radial-gradient(ellipse at bottom right, #260133, var(--nimiq-blue));\n  --nimiq-light-blue-bg: radial-gradient(ellipse at bottom right, #265dd7, var(--nimiq-light-blue));\n  --nimiq-gold-bg: radial-gradient(ellipse at bottom right, #ec991c, var(--nimiq-gold));\n  --nimiq-green-bg: radial-gradient(ellipse at bottom right, #41a38e, var(--nimiq-green));\n  --nimiq-orange-bg: radial-gradient(ellipse at bottom right, #fd6216, var(--nimiq-orange));\n  --nimiq-red-bg: radial-gradient(ellipse at bottom right, #cc3047, var(--nimiq-red));\n  --nimiq-blue-bg-darkened: radial-gradient(ellipse at bottom right, #180021, var(--nimiq-blue-darkened));\n  --nimiq-light-blue-bg-darkened: radial-gradient(ellipse at bottom right, #2355c4, var(--nimiq-light-blue-darkened));\n  --nimiq-gold-bg-darkened: radial-gradient(ellipse at bottom right, #e58a1b, var(--nimiq-gold-darkened));\n  --nimiq-green-bg-darkened: radial-gradient(ellipse at bottom right, #3d9988, var(--nimiq-green-darkened));\n  --nimiq-orange-bg-darkened: radial-gradient(ellipse at bottom right, #ea5200, var(--nimiq-orange-darkened));\n  --nimiq-red-bg-darkened: radial-gradient(ellipse at bottom right, #bf2d46, var(--nimiq-red-darkened));\n}\n\nbutton {\n  border-radius: 100px;\n  border: 0 none;\n  padding: 12px 20px;\n  margin: 0;\n  display: -ms-flexbox;\n  display: flex;\n  -ms-flex-direction: row;\n  flex-direction: row;\n  -ms-flex-align: center;\n  align-items: center;\n  font-size: 16px;\n  font-family: -apple-system, BlinkMacSystemFont, \'Segoe UI\', Roboto, Oxygen, Ubuntu, Cantarell, \'Open Sans\',\n    \'Helvetica Neue\', sans-serif;\n  cursor: pointer;\n  position: relative;\n  -webkit-box-shadow: none;\n  box-shadow: none;\n  -webkit-transition: all 300ms cubic-bezier(0.25, 0, 0, 1);\n  transition: all 300ms cubic-bezier(0.25, 0, 0, 1);\n}\n\nbutton:after {\n  content: \'\';\n  position: absolute;\n  left: -5px;\n  top: -5px;\n  right: -5px;\n  bottom: -5px;\n  border: 2px solid rgba(5, 130, 202, 0.5); /* Based on Nimiq Light Blue */\n  border-radius: 500px;\n  opacity: 0;\n}\n\nbutton:focus {\n  outline: none;\n}\n\nbutton:focus:not([disabled]):after {\n  opacity: 1;\n}\n\nbutton:hover {\n  -webkit-transform: translateY(-2px);\n  transform: translateY(-2px);\n  -webkit-box-shadow: 0 0.6rem 1rem rgba(0, 0, 0, 0.1);\n  box-shadow: 0 0.6rem 1rem rgba(0, 0, 0, 0.1);\n}\n\nbutton[disabled] {\n  background: var(--nimiq-gray) !important;\n  -webkit-box-shadow: none !important;\n  box-shadow: none !important;\n  -webkit-transform: none !important;\n  transform: none !important;\n  color: var(--nimiq-blue) !important;\n  cursor: not-allowed;\n  opacity: 0.4;\n}\n\nbutton[disabled] > .nim-icon {\n  color: var(--nimiq-blue);\n}\n\n.nim-logo {\n  width: 24px;\n  height: 24px;\n  margin-right: 12px;\n}\n\n/* Neutral theme */\n.nim-theme-neutral {\n  color: var(--nimiq-blue);\n  background-color: var(--nimiq-light-gray);\n}\n\n.nim-theme-neutral:hover {\n  background-color: var(--nimiq-gray);\n  -webkit-box-shadow: none;\n  box-shadow: none;\n}\n\n.nim-theme-neutral > .nim-icon {\n  color: #e9b213;\n}\n\n/* Blue theme */\n.nim-theme-blue {\n  color: #ffffff;\n  background-image: var(--nimiq-blue-bg);\n}\n\n.nim-theme-blue:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-blue-bg-darkened);\n}\n\n.nim-theme-blue > .nim-icon {\n  color: var(--nimiq-gold);\n}\n\n/* Gold theme */\n.nim-theme-gold {\n  color: #ffffff;\n  background-image: var(--nimiq-gold-bg);\n}\n\n.nim-theme-gold:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-gold-bg-darkened);\n}\n\n.nim-theme-gold > .nim-icon {\n  color: #ffffff;\n}\n\n/* Light blue theme */\n.nim-theme-light-blue {\n  color: #ffffff;\n  background-image: var(--nimiq-light-blue-bg);\n}\n\n.nim-theme-light-blue:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-light-blue-bg-darkened);\n}\n\n.nim-theme-light-blue > .nim-icon {\n  color: #ffffff;\n}\n\n/* Green theme */\n.nim-theme-green {\n  color: #ffffff;\n  background-image: var(--nimiq-green-bg);\n}\n\n.nim-theme-green:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-green-bg-darkened);\n}\n\n.nim-theme-green > .nim-icon {\n  color: #ffffff;\n}\n\n/* Orange theme */\n.nim-theme-orange {\n  color: #ffffff;\n  background-image: var(--nimiq-orange-bg);\n}\n\n.nim-theme-orange:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-orange-bg-darkened);\n}\n\n.nim-theme-orange > .nim-icon {\n  color: #ffffff;\n}\n\n/* Red theme */\n.nim-theme-red {\n  color: #ffffff;\n  background-image: var(--nimiq-red-bg);\n}\n\n.nim-theme-red:hover {\n  color: #ffffff;\n  background-image: var(--nimiq-red-bg-darkened);\n}\n\n.nim-theme-red > .nim-icon {\n  color: #ffffff;\n}"; }
};

export { Checkout as nim_checkout };
